import uuid
from urllib.parse import parse_qs

from sqlalchemy import text

from app.lib import util, settings, grid


async def index(scope, receive, send):
    dom, body = util.create_page("Main Page", scope)
    content = body.add_tag('div', {'class': 'content read'})

    # a GET request resets the "globals"
    globs = util.get_page_globals(scope)
    if scope.get('method', '') == 'GET':
        globs['page'] = '1'
        globs['epage'] = '1'
        globs.pop('selected', None)

    # POST handler allows the user to change the pages of the grids and select
    # which "office" should be used to show "employees" for
    if scope.get('method', '') == 'POST':
        parsed = parse_qs(await util.read_body(receive))
        # don't process a form unless it has a valid csrf token
        if util.get_post_var(parsed, b'csrf') == globs.get('csrf'):
            form = util.get_post_var(parsed, b'form')

            if form == 'grid1':
                if b'page' in parsed:
                    globs['page'] = util.get_post_var(parsed, b'page')
                    globs.pop('selected', None)
                if b'action' in parsed:
                    row = parsed[b'action'][0].decode('utf-8')[4:]
                    emp_id = globs['grid1ids[' + row + ']']
                    globs['selected'] = emp_id
                    globs['epage'] = '1'

            if form == 'grid2':
                if b'page' in parsed:
                    globs['epage'] = util.get_post_var(parsed, b'page')

        else:   # warn about token failure
            util.msg(content, settings.TOKEN_MESSAGE, True)

    globs['csrf'] = str(uuid.uuid4())
    util.save_page_globals(scope, globs)

    content.add_tag('h2', {'style': 'text-align: center;'}, "Grid Test")

    # create a grid of offices
    # this could be inserted into a template or simply tagged onto the content structure as we go.
    grid1 = grid.Grid("grid1", globs['csrf'])
    grid1.query = 'select * from offices '
    grid1.fields = [
        grid.Field('officeCode', '', grid.FieldType.STRING),
        grid.Field('city', 'City', grid.FieldType.STRING),
        grid.Field('addressLine1', 'Address', grid.FieldType.STRING),
        grid.Field('addressLine2', ' ', grid.FieldType.STRING),
        grid.Field('state', 'State', grid.FieldType.STRING),
        grid.Field('country', 'Country', grid.FieldType.STRING),
        grid.Field('postalCode', 'Post code', grid.FieldType.STRING),
        grid.Field('territory', 'Terrirory', grid.FieldType.STRING),
        grid.Field('phone', 'Phone', grid.FieldType.STRING),
    ]
    grid1.order = 'officeCode'
    grid1.idField = 'officeCode'
    grid1.page = int(globs['page'])
    grid1.actions = ['View']
    lv = grid1.populate(scope, content, 'grid1ids')
    globs.update(lv)
    util.save_page_globals(scope, globs)

    # First grid is inserted into our page, so check to see if we need to insert another grid
    content.add_tag('br').add_tag('br')
    if globs.get('selected', None):
        conn = scope['state']['conn']
        rs = conn.execute(text('SELECT officeCode,city,addressLine1,addressLine2,state,' +
                               'country,postalCode,territory,phone ' +
                               'FROM offices ' +
                               ' where officeCode = :id'), {'id': globs['selected']}).mappings().all()[0]
        content.add_tag('p', {}, str(rs['city']) + str(rs['state']) +
                        ' ' + str(rs['territory']) + ' ' + str(rs['phone']))
        grid2 = grid.Grid("grid2", globs['csrf'])
        grid2.query = "SELECT * FROM `employees` where officeCode = :oc"
        grid2.fields = [
            grid.Field('employeeNumber', 'Roll', grid.FieldType.STRING),
            grid.Field('firstName', 'Name', grid.FieldType.STRING),
            grid.Field('lastName', ' ', grid.FieldType.STRING),
            grid.Field('extension', 'EXT', grid.FieldType.STRING),
            grid.Field('email', 'Email', grid.FieldType.STRING),
            grid.Field('jobTitle', 'Role', grid.FieldType.STRING),
        ]
        grid2.params = {'oc': globs['selected']}
        grid2.order = 'employeeNumber'
        grid2.idField = 'employeeNumber'
        grid2.page = int(globs['epage'])
        grid2.rpp = 3
        grid2.populate(scope, content, 'g2id')

    content.add_tag('br').add_tag('br')
    util.dump_scope(content, scope)

    await util.render_page(scope, send, dom)
