from urllib.parse import parse_qs

import bcrypt
import datetime
from sqlalchemy import text

from app.lib import settings
from app.lib import util

# this example uses a password form, but could use some other form of authentication instead
async def index(scope, receive, send):
    dom, body = util.create_page("Log In", scope)
    content = body.add_tag('div', {'class': 'content'})

    if scope.get('method', '') == 'POST':
        post = await util.read_body(receive)
        parsed = parse_qs(post)
        if parsed != {} and parsed.get(b'user', [None])[0] and parsed.get(b'pass', [None])[0]:
            conn = scope['state']['conn']
            urs = conn.execute(text("SELECT * FROM " + settings.DB_CRED['USERTAB'] +
                                    " WHERE username = :user"), {'user': parsed[b'user'][0]})
            if urs:
                urs = urs.mappings().first()
            if urs and bcrypt.checkpw(parsed[b'pass'][0],
                                      bytes((urs['pass'].strip()), 'utf-8')):
                expires = datetime.datetime.now()
                expires = expires + datetime.timedelta(seconds=settings.MAX_SESS_TIME)
                conn.execute(text("update " + settings.DB_CRED['USERTAB'] +
                                  " set session = :sess, expires = :expires where username = :user"),
                             {'user': parsed[b'user'][0], 'sess': scope['state']['session'], 'expires': expires})
                conn.commit()

                parsed = parse_qs(scope['query_string'])
                redir = parsed.get(b'redir', [b'/'])[0]
                if redir in [('/' + settings.LOGOUT_PAGE).encode('utf-8'),
                             ('/' + settings.LOGIN_PAGE).encode('utf-8')]:
                    redir = b'/'
                await send({
                    'type': 'http.response.start',
                    'status': 302,
                    'headers': [(b'Location', settings.APP_ROOT.encode('utf-8') + redir),
                                (b'Set-Cookie', ('sess= ' + scope['state']['session'] + '; path = /').encode('utf-8'))]

                })
                await send({
                    'type': 'http.response.body',
                    'body': b''
                })
                return
    util.import_template(content, 'login.xml')
    content.add_tag('br').add_tag('br')
    
    util.dump_scope(content, scope)

    await util.render_page(scope, send, dom)
